"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateNG = void 0;
var utils_1 = require("./utils");
var is_acyclic_graph_1 = require("./is-acyclic-graph");
var nested_graph_1 = require("./nested-graph");
function generateNGHelper(opts) {
    var nestedGraph = new nested_graph_1.NestedGraph(new Array(opts.nodeCount).fill(null).map(function (_, i) { return ({
        id: i.toString()
    }); }));
    var _loop_1 = function (node) {
        if (utils_1.eventHappened(opts.nestedNessRatio)) {
            var otherNodes = nestedGraph
                .getAllNodes()
                .filter(function (n) { return n.id !== node.id && !nestedGraph.isAncestor(n.id, node.id); });
            if (otherNodes.length > 0) {
                nestedGraph.updateNode(node.id, __assign(__assign({}, node), { parentId: otherNodes[utils_1.randomNumber(otherNodes.length - 1)].id }));
            }
        }
    };
    // assign parents to each node based on nestedNessRatio
    for (var _i = 0, _a = nestedGraph.getAllNodes(); _i < _a.length; _i++) {
        var node = _a[_i];
        _loop_1(node);
    }
    // generate links between nodes based on outBoundLinkRatio
    var links = [];
    for (var _b = 0, _c = nestedGraph.getAllNodes(); _b < _c.length; _b++) {
        var node = _c[_b];
        var connectableNodes = nestedGraph.getAllNodesFromOtherSubTrees(node.id);
        for (var _d = 0, connectableNodes_1 = connectableNodes; _d < connectableNodes_1.length; _d++) {
            var connectable = connectableNodes_1[_d];
            if (utils_1.eventHappened(opts.outBoundLinkRatio) &&
                (opts.loopsAllowed ||
                    is_acyclic_graph_1.isAcyclicGraph(links.concat([{ sourceId: node.id, targetId: connectable.id }])))) {
                links.push({ sourceId: node.id, targetId: connectable.id });
            }
        }
    }
    nestedGraph.setLinks(links);
    return nestedGraph;
}
var defaultGraphConfig = {
    nodeCount: 10,
    nestedNessRatio: 0.1,
    outBoundLinkRatio: 0.1,
    loopsAllowed: false
};
function generateNG(opts) {
    if (opts) {
        if (opts.nodeCount !== undefined && opts.nodeCount <= 0) {
            throw new Error('nodeCount must be greater than zero');
        }
        if (opts.nestedNessRatio !== undefined &&
            !(opts.nestedNessRatio >= 0 && opts.nestedNessRatio <= 1.0)) {
            throw new Error('nestedNessRatio must be in the range [0, 1.0]');
        }
        if (opts.outBoundLinkRatio !== undefined &&
            !(opts.outBoundLinkRatio >= 0 && opts.outBoundLinkRatio <= 1.0)) {
            throw new Error('outBoundLinkRatio must be in the range [0, 1.0]');
        }
    }
    return generateNGHelper({
        nodeCount: opts.nodeCount !== undefined
            ? opts.nodeCount
            : defaultGraphConfig.nodeCount,
        nestedNessRatio: opts.nestedNessRatio !== undefined
            ? opts.nestedNessRatio
            : defaultGraphConfig.nestedNessRatio,
        outBoundLinkRatio: opts.outBoundLinkRatio !== undefined
            ? opts.outBoundLinkRatio
            : defaultGraphConfig.outBoundLinkRatio,
        loopsAllowed: opts.loopsAllowed !== undefined
            ? opts.loopsAllowed
            : defaultGraphConfig.loopsAllowed
    });
}
exports.generateNG = generateNG;
