"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var index_1 = require("./index");
var is_acyclic_graph_1 = require("./is-acyclic-graph");
describe('generateNestedGraph', function () {
    it('should generate random graph', function () {
        var _a = index_1.generateNestedGraph(), nodes = _a.nodes, links = _a.links;
        expect(nodes.length).toBeGreaterThan(0);
        expect(links.length).toBeGreaterThan(0);
        expect(is_acyclic_graph_1.isAcyclicGraph(links)).toBeTruthy();
    });
    it('should throw error on invalid inputs', function () {
        expect(function () { return index_1.generateNestedGraph({ nodeCount: -1 }); }).toThrowError(new Error('nodeCount must be greater than zero'));
        expect(function () { return index_1.generateNestedGraph({ nodeCount: 0 }); }).toThrow(new Error('nodeCount must be greater than zero'));
        expect(function () { return index_1.generateNestedGraph({ nestedNessRatio: -1 }); }).toThrowError(new Error('nestedNessRatio must be in the range [0, 1.0]'));
        expect(function () { return index_1.generateNestedGraph({ nestedNessRatio: 1.5 }); }).toThrow(new Error('nestedNessRatio must be in the range [0, 1.0]'));
        expect(function () { return index_1.generateNestedGraph({ outBoundLinkRatio: -1 }); }).toThrow(new Error('outBoundLinkRatio must be in the range [0, 1.0]'));
        expect(function () { return index_1.generateNestedGraph({ outBoundLinkRatio: 1.5 }); }).toThrow(new Error('outBoundLinkRatio must be in the range [0, 1.0]'));
    });
    it('some graphs generated should be cyclic when loopsAllowed flag is true', function () {
        /**
         * The reason why some graphs rather than all graphs are cyclic because the number
         * of links produced may not be enough to form a cycle in the graph.
         */
        var nodeCount = 5;
        while (nodeCount++ < 50) {
            var outBoundLinkRatio = 0;
            while (outBoundLinkRatio <= 1) {
                var iteration = 0;
                var cyclicGraphFound = false;
                while (!cyclicGraphFound && iteration++ < 100) {
                    var links = index_1.generateNestedGraph({
                        loopsAllowed: true,
                        nodeCount: nodeCount,
                        outBoundLinkRatio: outBoundLinkRatio
                    }).links;
                    cyclicGraphFound = !is_acyclic_graph_1.isAcyclicGraph(links);
                }
                if (outBoundLinkRatio > 0) {
                    expect(cyclicGraphFound).toBeTruthy();
                }
                else {
                    expect(cyclicGraphFound).toBeFalsy();
                }
                outBoundLinkRatio += 0.1;
            }
        }
    });
    it('graph should have some container elements if nestedNessRatio is greater than zero', function () {
        var nodeCount = 1;
        while (nodeCount++ < 30) {
            var nestedNessRatio = 0;
            while (nestedNessRatio <= 1) {
                var iteration = 0;
                var containerElementsFound = false;
                var _loop_1 = function () {
                    var nodes = index_1.generateNestedGraph({
                        nodeCount: nodeCount,
                        nestedNessRatio: nestedNessRatio
                    }).nodes;
                    containerElementsFound = nodes.some(function (n) {
                        return n.parentId !== undefined &&
                            n.id !== n.parentId &&
                            nodes.some(function (a) { return a.id === n.parentId; });
                    });
                };
                while (!containerElementsFound && iteration++ < 100) {
                    _loop_1();
                }
                if (nodeCount > 1 && nestedNessRatio > 0) {
                    expect(containerElementsFound).toBeTruthy();
                }
                else {
                    expect(containerElementsFound).toBeFalsy();
                }
                nestedNessRatio += 0.1;
            }
        }
    });
});
