"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NestedGraph = void 0;
var NestedGraph = /** @class */ (function () {
    function NestedGraph(nodes, links) {
        if (nodes === void 0) { nodes = []; }
        if (links === void 0) { links = []; }
        this.nodesMap = {};
        this.setNodes(nodes);
        this.links = links;
    }
    NestedGraph.prototype.setNodes = function (nodes) {
        var _this = this;
        this.nodesMap = {};
        nodes.forEach(function (n) { return (_this.nodesMap[n.id] = n); });
    };
    NestedGraph.prototype.updateNode = function (nodeId, node) {
        this.nodesMap[nodeId] = node;
    };
    NestedGraph.prototype.setLinks = function (links) {
        this.links = links;
    };
    NestedGraph.prototype.findNode = function (nodeId) {
        return this.nodesMap[nodeId];
    };
    NestedGraph.prototype.getAllNodes = function () {
        var _this = this;
        return Object.keys(this.nodesMap).map(function (k) { return _this.nodesMap[k]; });
    };
    NestedGraph.prototype.getAllLinks = function () {
        return this.links;
    };
    NestedGraph.prototype.isAncestor = function (nodeId, candidateNodeId) {
        var node = this.findNode(nodeId);
        return node && node.parentId !== undefined
            ? node.parentId === candidateNodeId ||
                this.isAncestor(node.parentId, candidateNodeId)
            : false;
    };
    NestedGraph.prototype.findRootNode = function (nodeId) {
        var node = this.findNode(nodeId);
        if (node) {
            if (node.parentId) {
                this.findRootNode(node.parentId);
            }
            else {
                return node;
            }
        }
        else {
            return undefined;
        }
    };
    NestedGraph.prototype.getAllNodesInSubGraph = function (rootNodeId) {
        var _this = this;
        var rootNode = this.findNode(rootNodeId);
        return rootNode
            ? this.getAllNodes()
                .filter(function (n) { return _this.isAncestor(n.id, rootNodeId); })
                .concat([rootNode])
            : [];
    };
    NestedGraph.prototype.getAllNodesFromOtherSubTrees = function (nodeId) {
        var rootNode = this.findRootNode(nodeId);
        if (rootNode) {
            var nodeIdsSetFromSubTree_1 = this.getAllNodesInSubGraph(rootNode.id)
                .map(function (n) { return n.id; })
                .reduce(function (acc, cur) {
                acc[cur] = true;
                return acc;
            }, {});
            return this.getAllNodes().filter(function (n) { return !nodeIdsSetFromSubTree_1[n.id]; });
        }
        else {
            return [];
        }
    };
    return NestedGraph;
}());
exports.NestedGraph = NestedGraph;
